@@ -70,7 +70,7 @@ sub parse_dpkg_lines {
                 $logger->warn($msg) if $check eq 'no';
             }
         }
-        elsif ( $l =~ /^([\w\-]+)\s*:/ ) {  # keyword:
+        elsif ( $l =~ m{^([\w\-]+)\s*:(?!//)} ) {  # keyword: but not http://
             my ($field,$text) = split /\s*:\s*/,$l,2 ;
             $text =~ s/\s+$//;
             $key = $field ;
@@ -98,9 +98,9 @@ sub parse_dpkg_lines {
         } 
         elsif ( $l =~ /^\s+\.$/) {   # line with a single dot
             $logger->trace("dot line: adding blank line to field $key");
-            _store_line_and_comments($store_list_ref,$file_path,"",$check,$line_nb, $handle_garbage, \@comments) ;
+            _store_line_and_comments($store_list_ref,$file_path,".",$check,$line_nb, $handle_garbage, \@comments) ;
         }
-        elsif ( $l =~ s/^\s//) {     # non empty line
+        elsif ( $l =~ /^\s/) {     # non empty line
             $logger->trace("text line: adding '$l' to field $key");
             _store_line_and_comments($store_list_ref,$file_path,$l , $check,$line_nb, $handle_garbage, \@comments);
         }
@@ -154,6 +154,7 @@ sub parse_dpkg_lines {
 sub _store_line_and_comments ($store_ref,$file_path,$line,$check,$line_nb, $handle_garbage, $comments) {
 
     if (defined $store_ref) {
+        substr $line,0,1,''; # remove first char which shows field continuation
         push $store_ref->@* , [ $line , $line_nb, '', $comments->@* ]
     }
     elsif ($handle_garbage) {
