@@ -14,7 +14,7 @@ use Path::Tiny;
 use Data::Dumper;
 
 use Config::Model::DumpAsData;
-use Dpkg::Copyright::Scanner qw/scan_files __squash __pack_files __pack_copyright
+use Dpkg::Copyright::Scanner qw/scan_files __squash_tree_of_copyright_ids __pack_files __pack_copyright
                                 __create_tree_leaf_from_paths __from_copyright_structure/;
 use Scalar::Util qw/weaken/;
 use Storable qw/dclone/;
@@ -157,6 +157,7 @@ sub update ($self, %args) {
         # prepare to be able to put back data matching an existing dir
         if ($old_path eq '*' or ($old_path =~ m!(.*)/\*$! and $current_dir->child($1)->is_dir )) {
             $preserved_path{$old_path} = delete $old_split_files{$old_path};
+            $self->_say( "Note: preserving '$old_path'" );
         }
         else {
             $self->_say( "Note: '$old_path' was removed (or excluded) from new upstream source" );
@@ -167,7 +168,7 @@ sub update ($self, %args) {
 
 
     # implode files entries with same data index
-    __squash(\%new_split_files, \@new_copyrights_by_id) ;
+    __squash_tree_of_copyright_ids(\%new_split_files, \@new_copyrights_by_id) ;
 
     # pack files by copyright id
     my @packed = __pack_files(\%new_split_files);
@@ -289,9 +290,15 @@ sub update ($self, %args) {
         $files_obj->fetch_with_id($old_path)->load_data( $preserved_path{$old_path} );
     }
 
-    # put back debian data
+    # put back debian data if file or dir is still present
     foreach my $deb_path (sort keys %debian_paths) {
-        $files_obj->fetch_with_id($deb_path)->load_data( $debian_paths{$deb_path} );
+        my $target = $deb_path =~ m!(.*)/\*$! ? $1 : $deb_path;
+        if ($current_dir->child($target)->exists) {
+            $files_obj->fetch_with_id($deb_path)->load_data( $debian_paths{$deb_path} );
+        }
+        else {
+            $self->_say("Note: dropping obsolete entry $deb_path");
+        }
     }
 
     $self->_apply_fix_scan_copyright_file($current_dir) ;
@@ -388,6 +395,15 @@ In debian package directory:
 
 Note: this command is experimental.
 
+=head1 Debian copyright data
+
+The C<Files: debian/*> section from C<debian/copyright> is often the
+only place containing copyright information for the files created by
+Debian maintainer. So all C<Files> entries beginning with C<debian/>
+are preserved during update. However, entries not matching an existing
+file or directory are removed.
+
+
 =head1 Tweak results
 
 Results can be tweaked either by:
