@@ -35,8 +35,11 @@ my $madison_host = 'api.ftp-master.debian.org';
 my $madison_endpoint = "https://$madison_host/madison";
 
 # list of virtual packages
-# See https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.txt
-# updated from 20 Feb 2019 version
+# See https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.yaml
+# Last update: 2021-01-17
+# wget -q -O- https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.yaml | \
+# perl -MYAML -e 'BEGIN {local $/=undef; $y = <>;} $yaml=Load($y); printf (" %s\n", $_->{name}) foreach @{$yaml->{virtualPackages}};' | \
+# sort -u
 my @virtual_list = qw/
  adventure
  audio-mixer
@@ -49,6 +52,7 @@ my @virtual_list = qw/
  dbus-session-bus
  debconf-2.0
  default-dbus-session-bus
+ default-logind
  dhcp-client
  dict-client
  dict-server
@@ -90,6 +94,7 @@ my @virtual_list = qw/
  lambdamoo-server
  libc-dev
  linux-kernel-log-daemon
+ logind
  lzh-archiver
  mail-reader
  mail-transport-agent
@@ -142,6 +147,7 @@ my @virtual_list = qw/
 # other less official virtual packages
 push @virtual_list, qw/
  debhelper-compat
+ libkvm-dev
  libgl-dev
  libqt5opengl5-desktop-dev
  libtiff-dev
@@ -154,6 +160,12 @@ push @virtual_list, qw/
 
 my %virtual_hash = map {( $_ => 1); } @virtual_list;
 
+sub _is_virtual ($pkg) {
+    return 1 if $virtual_hash{$pkg};
+    return 1 if $pkg =~ /^dh-sequence-[a-z0-9-]+$/;
+    return 0;
+}
+
 my %debian_map;
 my $version = \%Module::CoreList::version;
 
@@ -743,7 +755,7 @@ sub check_or_fix_pkg_name {
     }
     else {
         my @res = $self->get_available_version(  $pkg );
-		if ( @res == 0 and not $virtual_hash{$pkg}) {
+		if ( @res == 0 and not _is_virtual($pkg)) {
 			# no version found for $pkg
 			# don't know how to distinguish virtual package from source package
 			$logger->debug("unknown package $pkg");
@@ -843,8 +855,8 @@ sub get_available_version {
 
     $logger->debug("called on $pkg_name");
 
-    # don't query info for known virtual package
-    if ($virtual_hash{$pkg_name}) {
+    # don't query info for known or calculated virtual package
+    if (_is_virtual($pkg_name)) {
         $logger->debug("$pkg_name is a known virtual package");
         return ();
     }
@@ -901,7 +913,7 @@ sub cache_info_from_madison {
     my @needed;
 
     foreach my $pkg_name (@pkg_names) {
-        next if $virtual_hash{$pkg_name} ; # skip known virtual package
+        next if _is_virtual($pkg_name) ; # skip known or calculated virtual package
         my ($time,@res) = split / /, ($cache{$pkg_name} || '');
         if (defined $time and $time =~ /^\d+$/ and $time > $cache_expire_date) {
             $logger->debug("using cached info for $pkg_name");
@@ -1036,7 +1048,7 @@ syntax as described in http://www.debian.org/doc/debian-policy/ch-relationships.
 Whether the version specified with C<< > >> or C<< >= >> is necessary.
 This module checks with Debian server whether older versions can be
 found in Debian old-stable or not. If no older version can be found, a
-warning is issued (unless the package is known to be virtual)
+warning is issued (unless the package is known or calculated to be virtual)
 
 =item *
 
