@@ -5,11 +5,17 @@ return [
   {
     'class_description' => 'Contains the list of files to be installed by L<dh_install>',
     'element' => [
-      'list',
+      'content',
       {
         'cargo' => {
           'type' => 'leaf',
-          'value_type' => 'uniline'
+          'value_type' => 'uniline',
+          'warn_if_match' => {
+            '(^|\\s)/' => {
+              'fix' => 's!(^|\\s)/!$1!g',
+              'msg' => 'path should not begin with \'/\''
+            }
+          }
         },
         'description' => 'a file or files to install. The end of the line tells the directory it
 should be installed in. The name of the files (or directories) to
@@ -22,7 +28,7 @@ Note that if you list exactly one filename or wildcard-pattern, with
 no explicit destination, then dh_install will automatically guess the
 destination to use, the same as if the --autodest option were used.
 
-See L<dh_install> for more details.
+See L<debhelper(7)> and L<dh_install> for more details.
 ',
         'duplicates' => 'warn',
         'type' => 'list'
@@ -32,9 +38,9 @@ See L<dh_install> for more details.
     'rw_config' => {
       'auto_create' => '1',
       'auto_delete' => '1',
-      'backend' => 'PlainFile',
+      'backend' => 'Dpkg::DebHelperFile',
       'config_dir' => 'debian',
-      'file' => '&index(-).install'
+      'file' => 'install'
     }
   }
 ]
