@@ -9,7 +9,9 @@ use Array::IntSpan;
 use Path::Tiny;
 use Time::localtime;
 use Carp;
+use TOML::Tiny qw/from_toml/;
 use YAML::XS qw/LoadFile/;
+use JSON;
 
 $YAML::XS::LoadBlessed = 0;
 
@@ -54,7 +56,7 @@ sub print_copyright ( %args ) {
     }
 
     # regroup %files hash: all leaves have same id -> wild card
-    __squash($split_files, $copyrights_by_id);
+    __squash_tree_of_copyright_ids($split_files, $copyrights_by_id);
 
     # pack files by copyright id
     my @packed = __pack_files($split_files);
@@ -118,6 +120,7 @@ EOR
 $default{check} = << 'EOR2' ;
     (^|/)\w+$                 # scripts, AUTHORS files
     |^README                   # README.* files
+    |META\d?.json                # Perl or Raku meta file
     |\.(                          # search for file suffix
         c(c|pp|xx)?              # c and c++
        |h(h|pp|xx)?              # header files for c and c++
@@ -136,9 +139,11 @@ $default{check} = << 'EOR2' ;
        |sh
        |php
        |py(|x)
+       |rs                       # rust
        |rb
+       |rs
        |java
-       |js
+       |js(on)?
        |vala
        |el
        |sc(i|e)
@@ -149,6 +154,7 @@ $default{check} = << 'EOR2' ;
        |mod
        |m
        |md|markdown
+       |toml
        |tex
        |mli?
        |(c|l)?hs
@@ -159,11 +165,7 @@ EOR2
 # cleanup the regexp
 map { s/#.*\n//g;  s/[\s\n]+//g; } values %default;
 
-# option to skip UNKNOWN ?
-# load a file to override some entries ?
-sub scan_files ( %args ) {
-    $quiet = $args{quiet} // 0;
-
+sub _get_data_from_files  ( %args ) {
     my $current_dir = $args{from_dir} || path('.');
 
     my %regexps = %default; # default is needed during tests
@@ -203,99 +205,25 @@ sub scan_files ( %args ) {
         $pipe->close or die $! ? "Error closing licensecheck pipe: $!\n"
                                : "Exit status ".$?." $cmd\n";
     }
+    return _trim_lines (\@lines, \%regexps);
+}
 
-    my $fill_blank_data = __load_fill_blank_data($current_dir);
-
-    my %copyrights ;
-    my $files = {};
-    my $id = 1;
-    my @no_info_list;
+sub _trim_lines ($lines, $regexps) {
+    my @kept_lines;
     my @skipped;
-
-    foreach my $line (sort @lines) {
+    foreach my $line (sort $lines->@*) {
         chomp $line;
-        # say "found: $line";
         my ($f,$l,$c) = split /\t/, $line;
         $f =~ s!\./!!;
-        next if $files->{$f}; # file already parsed
-
-        # skip copyright because there's no need to have recursive copyrights
-        # skip changelog because it's often contains log entries beginning with copyright
-        # and it's a collective work by nature.
-        next if $f =~  m!debian/(copyright|changelog)!;
-
-        # skip copyright infos extracted from license files when they
-        # come from the author of the license and not from the project
-        # author
-        if ($f =~  m/^COPYING|LICENSE$/) {
-            if ($c =~ /Free Software Foundation/) { next; }
-            if ($c =~ /Perl Foundation/) { next; }
-        }
-
-
-        # skip license files without copyright information (usually plain license)
-        next if $f =~  m/^COPYING|LICENSE$/ && $c =~ /No copyright/;
 
         # filter out files user does not want checked
-        if ( $f !~ $regexps{check} ) {
+        if ( $f !~ $regexps->{check} ) {
             # the ignore test should not be usefull as the files are ignored by licensecheck
-            push @skipped, $f unless $f =~ $regexps{ignore};
-            next;
+            push @skipped, $f unless $f =~ $regexps->{ignore};
         }
-
-        $c =~ s/'//g;
-        $c =~ s/^&copy;\s*//g;
-        $c =~ s/\(c\)\s*//g;
-        $c =~ s/(?<=\b\d{4})\s*-\s*\d{4}(?=\s*-\s*(\d{4})\b)//g;
-        $c =~ s/(\d+)\s*-\s*(\d+)/$1-$2/g;
-        $c =~ s/\b(\d{4}),?\s+([\S^\d])/$1, $2/g;
-        $c =~ s/\s+by\s+//g;
-        $c =~ s/(\\n)*all\s+rights?\s+reserved\.?(\\n)*\s*//gi; # yes there are literal \n
-        $c = 'no-info-found' if $c =~ /^\*No/;
-        $c =~ s/\(r\)//g;
-        $c =~ s!^[\s,/*]|[\s,#/*-]+$!!g;
-        $c =~ s/--/-/g;
-        $c =~ s!\s+\*/\s+! !;
-        # libuv1 has copyright like "2000, -present"
-        $c =~ s![,\s]*-present!'-'.(localtime->year() + 1900)!e;
-        $c =~ s![\x00-\x1f].*!!; # cut off everything after and including the first non-printable
-
-        $c = __pack_copyright($c);
-
-        # this data overrides what's found in current files. This is done before
-        # the code that merge and coaslesce entries
-        my $fill_blank = __get_fill_blank($fill_blank_data, $f);
-
-        next if $fill_blank->{skip};
-
-        if ( $c =~ /no-info-found/ and $fill_blank->{copyright} ) {
-            $c = $fill_blank->{copyright};
+        else {
+            push @kept_lines, [ $f, $l, $c ];
         }
-        if ($fill_blank->{'override-copyright'}) {
-            _warn "Overriding path $f copyright info\n";
-            $c = $fill_blank->{'override-copyright'};
-        }
-        if ( $l =~/unknown/i and $fill_blank->{license} ) {
-            $l = $fill_blank->{license};
-        }
-        if ($fill_blank->{'override-license'}) {
-            _warn "Overriding path $f license info\n";
-            $l = $fill_blank->{'override-license'};
-        }
-
-        my @no_info_found;
-        if ( $c =~ /no-info-found/ ) {
-            push @no_info_found, 'copyright';
-        }
-        if ( $l =~/unknown/i ) {
-            push @no_info_found, 'license';
-        }
-
-        push @no_info_list , [$f , @no_info_found ] if @no_info_found;
-
-        my $has_info = @no_info_found < 2 ? 1 : 0;
-        $files->{$f} = $copyrights{$c}{$l} //= ($has_info ? $id++ : 0);
-        # say "Storing '$f' : '$c' '$l' has_info: $has_info id ".$files->{$f};
     }
 
     if (@skipped) {
@@ -308,6 +236,164 @@ sub scan_files ( %args ) {
         _warn $msg;
     }
 
+    return @kept_lines;
+}
+
+sub __skip_line ($f, $c, $l, $files, $fill_blank_data) {
+    # say "found: $line";
+    return 1 if $files->{$f};       # file already parsed
+
+    # skip copyright because there's no need to have recursive copyrights
+    # skip changelog because it often contains log entries beginning with copyright
+    # and it's a collective work by nature.
+    return 1 if $f =~  m!debian/(copyright|changelog)!;
+
+    # skip copyright infos extracted from license files when they
+    # come from the author of the license and not from the project
+    # author
+    if ($f =~  m/^COPYING|LICENSE$/) {
+        if ($c =~ /Free Software Foundation/) {
+            return 1;
+        }
+        if ($c =~ /Perl Foundation/) {
+            return 1;
+        }
+    }
+
+    # skip license files without copyright information (usually plain license)
+    return 1 if $f =~  m/^COPYING|LICENSE$/ && $c =~ /No copyright/;
+
+    # this data overrides what's found in current files. This is done before
+    # the code that merge and coalesce entries
+    my $fill_blank = __get_fill_blank($fill_blank_data, $f);
+
+    return 1 if $fill_blank->{skip};
+    return 0;
+}
+
+my $__extract_rust_info = sub ($file, $c, $l, $current_dir) {
+    my $cargotoml = $current_dir->child($file);
+
+    if ($cargotoml->is_file) {
+        my $toml = $cargotoml->slurp_utf8;
+        my $data = from_toml($toml);
+        my $license = $data->{'package'}{'license'};
+        my $authors = $data->{'package'}{'authors'} // [];
+        if (defined $license) {
+            # Cargo.toml spells AND and OR in capitals
+            # and allows / as a synonym to OR
+            $license =~ s! AND ! and !g;
+            $license =~ s! OR ! or !g;
+            $license =~ s!/! or !g;
+        }
+        return (join("\n ", @$authors) || $c, $license || $l);
+    }
+    return ($c, $l);
+};
+
+my $__extract_json_info = sub ($c_key, $l_key, $file, $c, $l, $current_dir) {
+    my $json_file = $current_dir->child($file);
+
+    if ($json_file->is_file) {
+        my $data = decode_json($json_file->slurp_utf8);
+        my @c_data = ref $data->{$c_key} ? $data->{$c_key}->@* : $data->{$c_key};
+        my @l_data = ref $data->{$l_key} ? $data->{$l_key}->@* : $data->{$l_key};
+        return (join("\n ", @c_data) || $c, join(" or ",@l_data) || $l);
+    }
+    return ($c, $l);
+};
+
+my %override = (
+    'Cargo.toml' => $__extract_rust_info,
+    'package.json' => sub {$__extract_json_info->('author','license',@_)},
+    'META.json' => sub {$__extract_json_info->('author','license',@_)},
+    'META6.json' => sub {$__extract_json_info->('authors','license',@_)},
+);
+
+sub __refine_line ($f, $c, $l, $current_dir, $files, $fill_blank_data) {
+    # override license information by scanning for TOML data
+    if ($override{$f}) {
+        ($c, $l) = $override{$f}->($f, $c, $l, $current_dir);
+        $l =~ s/MIT/Expat/g;
+    }
+
+    $c = __clean_copyright($c);
+
+    $c = __pack_copyright($c);
+
+    # Found in LICENSE files generated by Dist::Zilla
+    $l =~ s/(l?gpl)_(\d)/uc($1)."-$2"/e;
+    $l =~ s/_/./;
+    $l =~ s!GPL-1\+ and/or GPL-1!GPL-1+!;
+
+    my $fill_blank = __get_fill_blank($fill_blank_data, $f);
+    ($c, $l) = __apply_fill_blank($fill_blank, $f, $l, $c);
+    return ($f, $c, $l);
+}
+
+sub __store_line_info($f, $c, $l, $id, $files, $copyrights, $no_info_list) {
+    my @no_info_found;
+    if ( $c =~ /no-info-found/ ) {
+        push @no_info_found, 'copyright';
+    }
+    if ( $l =~/unknown/i ) {
+        push @no_info_found, 'license';
+    }
+
+    push $no_info_list->@* , [$f , @no_info_found ] if @no_info_found;
+
+    my $has_info = @no_info_found < 2 ? 1 : 0;
+    $files->{$f} = $copyrights->{$c}{$l} //= ($has_info ? $id++ : 0);
+    # say "Storing '$f' : '$c' '$l' has_info: $has_info id ".$files->{$f};
+    return $id;
+}
+
+# option to skip UNKNOWN ?
+# load a file to override some entries ?
+sub scan_files ( %args ) {
+    $quiet = $args{quiet} // 0;
+
+    my @line_refs = _get_data_from_files( %args );
+    my $current_dir = $args{from_dir} || path('.');
+
+    my $fill_blank_data = __load_fill_blank_data($current_dir);
+
+    my %copyrights ;
+    my $files = {};
+    my $id = 1;
+    my @no_info_list;
+
+    foreach my $ref (@line_refs) {
+        # say "found: $line";
+        my ($f,$l,$c) = $ref->@*;
+
+        next if __skip_line ($f, $c, $l, $files, $fill_blank_data);
+
+        ($f, $c, $l) = __refine_line ($f, $c, $l, $current_dir, $files, $fill_blank_data);
+
+        $id = __store_line_info($f, $c, $l, $id, $files, \%copyrights, \@no_info_list) ;
+    }
+
+    my @copyrights_by_id ;
+    foreach my $c (sort keys %copyrights) {
+        foreach my $l (sort keys $copyrights{$c}->%* ) {
+            my $id = $copyrights{$c}{$l};
+            $copyrights_by_id[$id] = __to_copyright_structure( $c, $l ) ;
+        }
+    }
+
+    _warn_user_about_problems ($files, $fill_blank_data, @no_info_list);
+
+    my $merged_c_info = __squash_copyrights_years (\@copyrights_by_id) ;
+
+    # replace the old ids with news ids
+    __swap_merged_ids($files, $merged_c_info);
+
+    # stop here for update ...
+    return ($files, \@copyrights_by_id) ;
+}
+
+sub _warn_user_about_problems ($files, $fill_blank_data, @no_info_list) {
     if (@no_info_list) {
         my $msg= "The following paths are missing information:\n";
         map {my ($p,@i) = $_->@*;
@@ -323,25 +409,10 @@ sub scan_files ( %args ) {
             .join("'\n- '",@notused)."'\n";
     }
 
-    my @copyrights_by_id ;
-    foreach my $c (sort keys %copyrights) {
-        foreach my $l (sort keys $copyrights{$c}->%* ) {
-            my $id = $copyrights{$c}{$l};
-            $copyrights_by_id[$id] = __to_copyright_structure( $c, $l ) ;
-        }
-    }
-
     warn "No copyright information found" unless keys %$files;
 
-    my $merged_c_info = __squash_copyrights_years (\@copyrights_by_id) ;
 
-    # replace the old ids with news ids
-    __swap_merged_ids($files, $merged_c_info);
-
-    # stop here for update ...
-    return ($files, \@copyrights_by_id) ;
 }
-
 sub __to_copyright_structure ($c, $l) {
     return {
         Copyright => $c,
@@ -371,6 +442,28 @@ sub __create_tree_leaf_from_paths ($h,$path,$value) {
     $h->{$last} = $value;
 }
 
+sub __clean_copyright ($c) {
+    $c =~ s/'//g;
+    $c =~ s/^&copy;\s*//g;
+    $c =~ s/\(c\)\s*//g;
+    $c =~ s/(?<=\b\d{4})\s*-\s*\d{4}(?=\s*-\s*(\d{4})\b)//g;
+    $c =~ s/(\d+)\s*-\s*(\d+)/$1-$2/g;
+    $c =~ s/\b(\d{4}),?\s+([\S^\d])/$1, $2/g;
+    $c =~ s/\s+by\s+//g;
+    $c =~ s/(\\n)*all\s+rights?\s+reserved\.?(\\n)*\s*//gi; # yes there are literal \n
+    $c = 'no-info-found' if $c =~ /^\*No/;
+    $c =~ s/\(r\)//g;
+    $c =~ s!^[\s,/*]|[\s,#/*-]+$!!g;
+    $c =~ s/--/-/g;
+    $c =~ s!\s+\*/\s+! !;
+    # libuv1 has copyright like "2000, -present"
+    $c =~ s![,\s]*-present!'-'.(localtime->year() + 1900)!e;
+    $c =~ s![\x00-\x1f].*!!; # cut off everything after and including the first non-printable
+    # cleanup markdown copyright
+    $c =~ s/\[([\w\s]+)\]\(mailto:([\w@.+-]+)\)/$1 <$2>/;
+    return $c;
+}
+
 sub __pack_copyright ($r) {
 
     return $r if $r eq 'no-info-found';
@@ -420,7 +513,7 @@ sub __pack_files ($h) {
         $a->[1] cmp $b->[1];
     };
 
-    return sort $sort_path @res ;
+    return (sort $sort_path @res) ;
 }
 
 sub __pack_dir ($h, $pack, @path) {
@@ -533,29 +626,33 @@ sub __coalesce_copyright_years($entries, $owners) {
     return @squashed_c;
 }
 
-# $h is a tree of hash matching the directory structure. Each leaf is a
-# copyright id. Each key is a file name in a directory (not the full path)
-sub __squash ($h, $copyrights_by_id, $p = '') {
-    my %count ;
-
-    # find main license info found in LICENSE or COPYING or README
-    #  file. LICENSE and COPYING are removed from the file list as
-    #  they specify the license author as copyright owner
-    my %main_infos;
+sub __find_main_license_info ($tree_of_ids) {
+    my %main_info;
+    foreach my $name (sort keys %$tree_of_ids) {
+        if ($override{$name}) {
+            my $id = $tree_of_ids->{$name};
+            next unless $id; # skip files without info
+            delete $tree_of_ids->{$name};
+            $main_info{$id} = 1;
+        };
+    }
     foreach my $info_name (qw/README LICENSE LICENCE COPYING/) {
         my $re = qr!^$info_name[.\w]*$!;
-        foreach my $name (sort keys %$h) {
+        foreach my $name (sort keys %$tree_of_ids) {
             if ($name =~ $re) {
-                my $id = $h->{$name};
-                delete $h->{$name} if $info_name ne 'README';
+                my $id = $tree_of_ids->{$name};
+                delete $tree_of_ids->{$name} if $info_name ne 'README';
                 next unless $id; # skip files without info
-                $main_infos{$id} = 1;
+                $main_info{$id} = 1;
             };
         }
     }
+    return \%main_info;
+}
 
+sub __find_main_license_id ($copyrights_by_id, $main_info) {
     my $main_license_id;
-    my @main_info_keys = sort keys %main_infos;
+    my @main_info_keys = sort keys %$main_info;
     if (@main_info_keys == 1) {
         $main_license_id = $main_info_keys[0];
     }
@@ -577,15 +674,19 @@ sub __squash ($h, $copyrights_by_id, $p = '') {
             );
         $main_license_id = $#$copyrights_by_id;
     }
+    return $main_license_id;
+}
 
+sub __update_main_license_from_files ($main_license_id, $tree_of_ids, $copyrights_by_id, $path) {
+    my %count ;
     # count the number of times each (c) info is used in this directory.
     # (including the main (c) info of each subdirectory)
-    foreach my $name (sort keys %$h) {
-        my $item = $h->{$name};
+    foreach my $name (sort keys %$tree_of_ids) {
+        my $item = $tree_of_ids->{$name};
         if (ref($item)) {
             # squash may return a plain id, or a hash with '*' => id ,
             # or a non squashable hash
-            $h->{$name} = __squash($item, $copyrights_by_id, $p.'/'.$name);
+            $tree_of_ids->{$name} = __squash_tree_of_copyright_ids($item, $copyrights_by_id, $path.'/'.$name);
         }
         my $id = (ref($item) and defined $item->{'*'}) ? $item->{'*'} : $item ;
 
@@ -608,12 +709,15 @@ sub __squash ($h, $copyrights_by_id, $p = '') {
             }
         }
     }
+    return $main_license_id;
+}
 
+sub __prune_files_represented_by_main_license ($main_license_id, $tree_of_ids) {
     # all files associated to the most used (c) info are deleted to
     # be represented by '*' entry
     if (defined $main_license_id) {
-        foreach my $name (sort keys %$h) {
-            my $item = $h->{$name};
+        foreach my $name (sort keys %$tree_of_ids) {
+            my $item = $tree_of_ids->{$name};
             if (ref($item) and defined $item->{'*'} and $item->{'*'} == $main_license_id) {
                 # rename item/* to item/. when covered by ./*
                 # this is a "weak" directory info which is handled specially
@@ -621,15 +725,30 @@ sub __squash ($h, $copyrights_by_id, $p = '') {
             }
             if (not ref ($item)) {
                 # delete file that is represented by '*' entry
-                delete $h->{$name} if $item == $main_license_id;
+                delete $tree_of_ids->{$name} if $item == $main_license_id;
             }
         }
 
         # here's the '*' file representing the most used (c) info
-        $h->{'*'} //= $main_license_id;
+        $tree_of_ids->{'*'} //= $main_license_id;
     }
+}
 
-    return $h;
+# $tree_of_ids is a tree of hash matching the directory structure. Each leaf is a
+# copyright id. Each key is a file name in a directory (not the full path)
+sub __squash_tree_of_copyright_ids ($tree_of_ids, $copyrights_by_id, $path = '') {
+    # find main license info found in LICENSE or COPYING or README
+    #  file. LICENSE and COPYING are removed from the file list as
+    #  they specify the license author as copyright owner
+    my $main_info = __find_main_license_info($tree_of_ids);
+
+    my $main_license_id = __find_main_license_id($copyrights_by_id, $main_info);
+
+    $main_license_id = __update_main_license_from_files ($main_license_id, $tree_of_ids, $copyrights_by_id, $path);
+
+    __prune_files_represented_by_main_license ($main_license_id, $tree_of_ids);
+
+    return $tree_of_ids;
 }
 
 sub __load_fill_blank_data ($current_dir) {
@@ -672,6 +791,23 @@ sub __get_fill_blank ($fbd,$file) {
     return {};
 }
 
+sub __apply_fill_blank($fill_blank, $f, $l, $c) {
+    if ( $c =~ /no-info-found/ and $fill_blank->{copyright} ) {
+        $c = $fill_blank->{copyright};
+    }
+    if ($fill_blank->{'override-copyright'}) {
+        _warn "Overriding path $f copyright info\n";
+        $c = $fill_blank->{'override-copyright'};
+    }
+    if ( $l =~/unknown/i and $fill_blank->{license} ) {
+        $l = $fill_blank->{license};
+    }
+    if ($fill_blank->{'override-license'}) {
+        _warn "Overriding path $f license info\n";
+        $l = $fill_blank->{'override-license'};
+    }
+    return ($c, $l);
+}
 1;
 
 __END__
